﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectBulldozer.Drawnings;

namespace ProjectBulldozer.CollectionGenericObjects;

/// <summary>
/// Реализация абстрактной компании
/// </summary>
public class Garage : AbstractCompany
{
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="picWidth"></param>
    /// <param name="picHeight"></param>
    /// <param name="collectoin"></param>
    public Garage(int picWidth, int picHeight, ICollectoinGenericObjects<DrawningDozer> collectoin) : base(picWidth, picHeight, collectoin)
    {

    }

    /// <summary>
    /// отрисовка парковки
    /// </summary>
    /// <param name="g"></param>
    protected override void DrawBackground(Graphics g)
    {
        int cntVertically = _pictureHeight / _placeSizeHeight; //Колличество мест по вертикали
        int cntHorizontally = _pictureWidth / _placeSizeWidth; //Колличество мест по горизонтали
        Pen pen = new(Color.FromArgb(185, 140, 0))
        {
            Width = 3
        };

        for (int i = 0; i < cntHorizontally; i++)
        {
            g.DrawLine(pen, i * _placeSizeWidth, 0, i * _placeSizeWidth, cntVertically * _placeSizeHeight);
            for (int j = 0; j < cntVertically + 1; j++)
            {
                g.DrawLine(pen, i * _placeSizeWidth, j * _placeSizeHeight, (i + 1) * _placeSizeWidth - 50, j * _placeSizeHeight);
            }
        }
    }

    /// <summary>
    /// выбор места на парковке
    /// </summary>
    protected override void SetObjectsPosition()
    {
        //Влево, вверх
        int width = _pictureWidth / _placeSizeWidth - 1;
        int height = _pictureHeight / _placeSizeHeight - 1;

        int placeHorizontally = width;
        int placeVertically = height;

        for (int i = 0; i < (_collection?.Count ?? 0); i++)
        {
            if (placeVertically < 0)
            {
                return;
            }
            if (_collection?.Get(i) != null)
            {
                _collection?.Get(i)?.SetPictureSize(_pictureWidth, _pictureHeight);
                _collection?.Get(i)?.SetPosition(_placeSizeWidth * placeHorizontally + 20, _placeSizeHeight * placeVertically + 20);
            }

            if (placeHorizontally > 0)
            {
                placeHorizontally--;
            }
            else
            {
                placeHorizontally = width;
                placeVertically--;
            }
        }
    }
}
