﻿using ProjectBulldozer.Exceptions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectBulldozer.CollectionGenericObjects;

/// <summary>
/// Параметрезированный набор объектов
/// </summary>
/// <typeparam name="T">Параметр: ограничение - ссылочный тип</typeparam>
public class ListGenericObjects<T> : ICollectoinGenericObjects<T> 
    where T : class
{
    /// <summary>
    /// Список объектов, которые храним
    /// </summary>
    private readonly List<T?> _collection;

    /// <summary>
    /// Максимально допустимое число объектов в списке
    /// </summary>
    private int _maxCount;

    public int Count => _collection.Count;

    public int MaxCount {
        get
        {
            return _collection.Count;
        }
        
        set 
        { 
            if (value > 0) 
            {  
                _maxCount = value; 
            } 
        } 
    }

    public CollectionType GetCollectionType => CollectionType.List;

    /// <summary>
    /// Конструктор
    /// </summary>
    public ListGenericObjects()
    {
        _collection = new();
    }

    public T? Get(int position)
    {
        // Проверка позиции.
        if (position < 0 || position >= Count)
        {
            throw new PositionOutOfCollectionException(position);
        }

        // Выброс ошибки, если выход за границы списка
        if (_collection[position] == null)
        {
            throw new ObjectNotFoundException(position);
        }

        return _collection[position];
    }
    public int Insert(T obj)
    {
        // Выброс ошибки, если переполнение
        if (Count == _maxCount)
        {
            throw new CollectionOverflowException(Count);
        }

        // Вставка в конец набора.
        _collection.Add(obj);
        return _collection.Count;
    }

    public int Insert(T obj, int position)
    {
        // Выброс ошибки, если переполнение
        if (Count == _maxCount)
        {
            throw new CollectionOverflowException(Count);
        }

        // Выброс ошибки, если выход за границы списка
        if (position >= Count || position < 0)
        {
            throw new PositionOutOfCollectionException(position);
        }

        // Вставка по позиции.
        _collection.Insert(position, obj);
        return position;
    }


    public T? Remove(int position)
    {
        // TODO Выброс ошибки, если выход за границы списка
        if (position >= Count || position < 0)
        {
            throw new PositionOutOfCollectionException(position);
        }

        // Удаление объекта из списка.
        T? obj = _collection[position];
        _collection[position] = null;
        return obj;
    }

    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < _collection.Count; ++i)
        {
            yield return _collection[i];
        }
    }
}
