﻿using ProjectBulldozer.Exceptions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectBulldozer.CollectionGenericObjects;

public class MassiveGenericObject<T> : ICollectoinGenericObjects<T> 
    where T : class
{
    /// <summary>
    /// Массив объектов, который храним
    /// </summary>
    private T?[] _collection;

    public int Count => _collection.Length;

    public int MaxCount 
    { 
        get
        {
            return _collection.Length;
        }
        set
        {
            if (value > 0)
            {
                if (_collection.Length > 0)
                {
                    Array.Resize(ref _collection, value);
                }
                else
                {
                    _collection = new T[value];
                }
            }
        }
    }

    public CollectionType GetCollectionType => CollectionType.Massive;

    /// <summary>
    /// Конструктор
    /// </summary>
    public MassiveGenericObject()
    {
        _collection = Array.Empty<T>();
    }

    public T? Get(int position)
    {
        //Проверка позиции
        // Выброс ошибки, если выход за границы массива
        if (position < 0 || position >= _collection.Length)
        {
            throw new PositionOutOfCollectionException(position);
        }

        // Выброс ошибки, если объект пустой
        if (position >= _collection.Length && _collection[position] == null)
        {
            throw new ObjectNotFoundException(position);
        }

        return _collection[position];
    }

    public int Insert(T obj)
    {
        //Вставка в свободное место набора
        for (int i = 0; i < Count; i++)
        {
            if (_collection[i] == null)
            {
                _collection[i] = obj;
                return i;
            }
        }

        // Выброс ошибки, если переполнение
        throw new CollectionOverflowException(Count);
    }

    public int Insert(T obj, int position)
    {
        //Проверка позиции
        if ((position < 0) || (position >= Count))
        {
            return -1;
        }

        // Выброс ошибки, если переполнение
        if (position < 0 || position >= _collection.Length)
        {
            throw new PositionOutOfCollectionException(position);
        }


        //Проверка, что элемент массива по этой позиции пустой, если нет, то ищется свободное место после этой
        //позиции и идёт вставка туда, если нет после, ищем до
        if (_collection[position] == null)
        {
            //Вставка
            _collection[position] = obj;
            return position;
        }
        int temp = position + 1;
        while (temp < Count)
        {
            if (_collection[temp] == null)
            {
                _collection[temp] = obj;
                return temp;
            }
            ++temp;
        }

        temp = position - 1;
        while (temp >= 0)
        {
            if (_collection[temp] == null)
            {
                _collection[temp] = obj;
                return temp;
            }
            --temp;
        }

        // Выброс ошибки, если выход за границы массива
        throw new CollectionOverflowException(Count);
    }

    public T? Remove(int position)
    {
        //Проверка позиции
        // Выброс ошибки, если выход за границы массива
        if ((position < 0) || (position >= Count))
        {
            throw new PositionOutOfCollectionException(position);
        }

        // Выброс ошибки, если объект пустой
        if (_collection[position] == null)
        {
            throw new ObjectNotFoundException(position);
        }

        //Удаление объекта из массива, присвоив элементу массива значение null
        T? elem = _collection[position];
        _collection[position] = null;
        return elem;
    }

    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < _collection.Length; ++i)
        {
            yield return _collection[i];
        }
    }
}
