﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectBulldozer.Entities;

namespace ProjectBulldozer.Drawnings;

/// <summary>
/// Расширение для класса EntityDozer
/// </summary>
public static class ExtentionDrawningDozer
{
    /// <summary>
    /// Разделитель строки для записи информации по объекту.
    /// </summary>
    private static readonly string _separatorForObject = ":";
    
    /// <summary>
    /// Создание объекта из строки.
    /// </summary>
    /// <param name="info"></param>
    /// <returns></returns>
    public static DrawningDozer? CreateDrawningDozer(this string info)
    {
        string[] strs = info.Split(_separatorForObject);
        EntityDozer? dozer = EntityBulldozer.CreateEntityBulldozer(strs);
        if (dozer != null) 
        {
            return new DrawningBulldozer(dozer);
        }

        dozer = EntityDozer.CreateEntityDozer(strs);
        if (dozer != null)
        {
            return new DrawningDozer(dozer);
        }

        return null;
    }

    /// <summary>
    /// Получение данных для сохранения в файл.
    /// </summary>
    /// <param name="drawningDozer"></param>
    /// <returns></returns>
    public static string GetDataForSave(this DrawningDozer drawningDozer)
    {
        string[]? array = drawningDozer?.EntityDozer?.GetStringRepresentation();

        if (array == null)
        {
            return string.Empty;
        }

        return string.Join(_separatorForObject, array);
    }
}
