﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectBulldozer.Entities;

/// <summary>
/// Класс-сущность "Бульдозер"
/// </summary>
public class EntityBulldozer : EntityDozer
{
    

    ///<summary>
    /// Признак (опция) наличие отвала(ковша)
    /// </summary>
    public bool Blade { get; private set; }

    ///<summary>
    /// Признак (опция) наличие гусеницы
    /// </summary>
    public bool Caterpillar { get; private set; }


    ///<summary>
    /// Дополнительный цвет 
    /// </summary>
    public Color AdditionalColor { get; private set; }

    public void SetAdditionalColor(Color color)
    {
        AdditionalColor = color;
    }

        ///<summary>
        ///Инициализация полей объекта-класса бульдозера
        /// </summary>
        ///<param name="speed">Скорость</param>
        ///<param name="weight">Вес</param>
        ///<param name="bodyColor">Основной цвет</param>
        ///<param name="additionalColor">Дополнительный цвет</param>
        ///<param name="blade">Признак наличия отвала</param>
        ///<param name="caterpillar">Признак наличия гусеницы</param>
    public EntityBulldozer(int speed, double weight, Color bodyColor, Color additionalColor, bool blade, bool caterpillar) : base(speed, weight, bodyColor)
    {
        AdditionalColor = additionalColor;
        Blade = blade;
        Caterpillar = caterpillar;
    }


    /// <summary>
    /// Получение строк со значениями свойств объекта класса-сущности.
    /// </summary>
    /// <returns></returns>
    public override string[] GetStringRepresentation()
    {
        return new[] { nameof(EntityBulldozer), Speed.ToString(), Weight.ToString(), Weight.ToString(), BodyColor.Name,
            AdditionalColor.Name, Blade.ToString(), Caterpillar.ToString() };
    }

    /// <summary>
    /// Создание объекта из массива строк.
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static EntityBulldozer? CreateEntityBulldozer(string[] strs)
    {
        if (strs.Length != 7 || strs[0] != nameof(EntityBulldozer))
        {
            return null;
        }
        return new EntityBulldozer(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]),
            Color.FromName(strs[4]), Convert.ToBoolean(strs[5]), Convert.ToBoolean(strs[6]));
    }
}
