﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectBulldozer.Entities;

/// <summary>
/// Класс-сущность "Бульдозер"
/// </summary>
public class EntityDozer
{
    ///<summary>
    /// Скорость
    /// </summary>
    public int Speed { get; private set; }

    ///<summary>
    /// Вес
    /// </summary>
    public double Weight { get; private set; }

    ///<summary>
    /// Основной цвет
    /// </summary>
    public Color BodyColor { get; private set; }

    public void SetBodyColor(Color color)
    {
        BodyColor = color;
    }

    ///<summary>
    /// Шаг перемещения бульдозера
    /// </summary>
    public double Step => Speed * 100 / Weight;

    ///<summary>
    /// Конструктор
    /// </summary>
    ///<param name="speed">Скорость</param>
    ///<param name="weight">Вес</param>
    ///<param name="bodyColor">Основной цвет</param>
    public EntityDozer(int speed, double weight, Color bodyColor) 
    {
        Speed = speed;
        Weight = weight;
        BodyColor = bodyColor;
    }

    /// <summary>
    /// Получение строк со значениями свойств объекта класса-сущности.
    /// </summary>
    /// <returns></returns>
    public virtual string[] GetStringRepresentation()
    {
        return new[] { nameof(EntityDozer), Speed.ToString(), Weight.ToString(), BodyColor.Name };
    }

    /// <summary>
    /// Создание объекта из массива строк.
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static EntityDozer? CreateEntityDozer(string[] strs)
    {
        if (strs.Length != 4 || strs[0] != nameof(EntityDozer)) return null;

        return new EntityDozer(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]));
    }
}
