﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ProjectBulldozer.Drawnings;
using ProjectBulldozer.Entities;

namespace ProjectBulldozer;

/// <summary>
/// Форма конигурации объекта
/// </summary>
public partial class FormBulldozerConfig : Form
{
    /// <summary>
    /// Объект прорисовка бульдозера
    /// </summary>
    private DrawningDozer? _dozer /*= null*/;

    /// <summary>
    /// Событие для передачи объекта
    /// </summary>
    public event Action<DrawningDozer>? _dozerDelegate;

    //private event Action<> = 

    /// <summary>
    /// Конструктор
    /// </summary>
    public FormBulldozerConfig()
    {
        InitializeComponent();
        panelGold.MouseDown += Panel_MouseDown;
        panelYellow.MouseDown += Panel_MouseDown;
        panelRed.MouseDown += Panel_MouseDown;
        panelOrange.MouseDown += Panel_MouseDown;
        panelCadetBlue.MouseDown += Panel_MouseDown;
        panelWhite.MouseDown += Panel_MouseDown;
        panelSilver.MouseDown += Panel_MouseDown;
        panelBlack.MouseDown += Panel_MouseDown;

        // buttonCancel.Click привязка анонимного метода через lambda с закрытием формы.
        buttonCancel.Click += (sender, e) => Close();


    }

    /// <summary>
    /// Привязка внешнего метода к событию
    /// </summary>
    /// <param name="dozerDelegate"></param>
    public void AddEvent(Action<DrawningDozer>? dozerDelegate)
    {
        _dozerDelegate += dozerDelegate;
    }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    private void DrawObject()
    {
        Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _dozer?.SetPictureSize(pictureBoxObject.Width, pictureBoxObject.Height);
        _dozer?.SetPosition(25, 30);
        _dozer?.DrawTransport(gr);
        pictureBoxObject.Image = bmp;
    }

    /// <summary>
    /// Передаём информацию при нажатии на Label
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelObject_MouseDown(object sender, MouseEventArgs e)
    {
        (sender as Label)?.DoDragDrop((sender as Label)?.Name ?? string.Empty,
            DragDropEffects.Move | DragDropEffects.Copy);
    }

    /// <summary>
    /// Проверка получаемой информации (её типа на соответствии требуему)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void PanelObject_DragEnter(object sender, DragEventArgs e)
    {
        e.Effect = e.Data?.GetDataPresent(DataFormats.Text) ?? false ? DragDropEffects.Copy : DragDropEffects.None;
    }

    /// <summary>
    /// Действия при приёме перетаскиваемой информации
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void PanelObject_DragDrop(object sender, DragEventArgs e)
    {
        switch (e.Data?.GetData(DataFormats.Text)?.ToString())
        {
            case "labelSimpleObject":
                _dozer = new DrawningDozer((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White);
                break;
            case "labelModifiedObject":
                _dozer = new DrawningBulldozer((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value,
                    Color.White, Color.Black, checkBoxBlade.Checked, checkBoxCaterpillar.Checked);
                break;
        }
        DrawObject();
    }

    /// <summary>
    /// Передаём информацию при нажатии на Panel
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void Panel_MouseDown(object? sender, MouseEventArgs e)
    {
        // Отправка цвета в Drag&Drop
        (sender as Control)?.DoDragDrop((sender as Control)?.BackColor,
            DragDropEffects.Move | DragDropEffects.Copy);
    }

    // Реализация логики смены цветов: основного и дополнительного (для смены объекта)

    /// <summary>
    /// Передача основного цвет
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelBodyColor_DragEnter(object sender, DragEventArgs e)
    {
        if (e.Data.GetDataPresent(typeof(Color)))
        {
            e.Effect = DragDropEffects.Copy;
        }
        else
        {
            e.Effect = DragDropEffects.None;
        }
    }

    /// <summary>
    /// Закраска основным цветом
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelBodyColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_dozer == null) return;
        _dozer.EntityDozer?.SetBodyColor((Color)e.Data.GetData(typeof(Color)));
        DrawObject();
    }

    /// <summary>
    /// Передача дополнительного цвета
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelAdditionalColor_DragEnter(object sender, DragEventArgs e)
    {
        if (_dozer is DrawningBulldozer)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
    }

    /// <summary>
    /// Расскраска дополнительным цветом
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelAdditionalColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_dozer == null) return;
        if (_dozer?.EntityDozer is EntityBulldozer entityBulldozer)
        {
            entityBulldozer.SetAdditionalColor((Color)e.Data.GetData(typeof(Color)));
        }
        DrawObject();
    }

    /// <summary>
    /// Передача объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAdd_Click(object sender, EventArgs e)
    {
        if (_dozer != null)
        {
            _dozerDelegate?.Invoke(_dozer);
            Close();
        }
    }
}
